/* file name  : parser/events/metaevents/MetaPropagationEvent.java
 * authors    : Guillaume Arnaud
 * created    : lun 22 mar 2004 17:30:41 CET
 * copyright  : 
 *
 * modifications:
 *
 */

package parser.propagation;

import fr.inria.oadymppac.parsing.gentra4cp.event.attributes.EventAttributes;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;

/** 
 * Class MetaPropagationEvent. This class uses events inherited from {@link PropagationEvent}
 * for building a meta propagation event. 
 * This event records informations during propagation process between two control events,
 * inherited from {@link EventAttributes}. 
 * The start and end conditions of each record are {@link ChoicePointEvent},
 * {@link SolutionEvent} and {@link RejectEvent}.
 * 
 * @author Guillaume Arnaud
 * @version 1.0
 */
public class MetaPropagationEvent{
	/**
	 * Constant about the status of the events. 
     * No specific status.
	 */
	public static final int NOTYPE=0;
	/**
	 * Constant about the status of the events. 
     * The propagation ends with a solution event.
	 */
	public static final int SOLUTION=1;
	/**
	 * Constant about the status of the events. 
     * The propagation ends with a reject event.
	 */
	public static final int REJECT=2;
	/**
	 * Constant about the status of the events. 
     * he propagation ends with a choice point event.
	 */    
	public static final int CHOICEPOINT=3;

	/**
	 * Total number of propagations.
	 */
	private int nbTotalPropagation=0;
	
	/** 
	 * The propagation event which ends this meta propagation event. It could be
     * a {@link parser.events.ChoicePointEvent ChoicePointEvent}, a {@link parser.events.SolutionEvent SolutionEvent} or a {@link  parser.events.RejectEvent RejectEvent}.
	 */
	private EventAttributes event;

	/** 
	 * Type of the meta propagation event. The possible values are {@link #NOTYPE NOTYPE}, {@link #CHOICEPOINT CHOICEPOINT}, {@link #REJECT REJECT} and {@link #SOLUTION SOLUTION}
	 */
	private int type; 
	
	/** 
	 *  Names of the constraints actived during the propagation.
	 */
	private Vector constraintNames;

	/** 
	 * Hashtable which records for each type of propagation event (inherited from {@link
     * PropagationEvent}) their number of occurences. Propagation event
     * is identified by a string ("reduce", "suspend" and so on) which depends on the propagation
     * manager {@link MetaPropagationEventManager}.
	 */
	private Hashtable totalEvents=new Hashtable();

	/** 
	 * Hashtable which records for each type of propagation event (inherited from {@link
     * PropagationEvent}) a hashtable of filtered constraints. These constraints are
     * queried by a user and are associated with their number of occurences during the
     * propagation process.
     */
	private Hashtable filteredConstraints=new Hashtable();

	/** 
	 * 
	 */
	public MetaPropagationEvent() {}
    
	public MetaPropagationEvent(EventAttributes e, int nbTotalPropagation) {
		this.nbTotalPropagation=nbTotalPropagation;
		event=e;
		type=NOTYPE;
	}
	public MetaPropagationEvent(int type, int nbTotalPropagation) {
		this.nbTotalPropagation=nbTotalPropagation;
		event=null;
		if(type>=NOTYPE&&type<=CHOICEPOINT) this.type=type;
		else this.type=NOTYPE;
	}
	public MetaPropagationEvent(EventAttributes e, int type, int nbTotalPropagation,Vector constraintNames) {
		this.nbTotalPropagation=nbTotalPropagation;
		event=e;
		if(type>=NOTYPE&&type<=CHOICEPOINT) this.type=type;
		else this.type=NOTYPE;
		this.constraintNames=constraintNames;
	}

	/**
	 * Gets the event.
	 * @return EventAttributes event propagation.
     * @see #event
	 */
	public EventAttributes getEvent() { return event;}
	
	/**
	 * Gets the number of propagation.
	 * @return nbTotalPropagation the number total of propagation.
     * @see #nbTotalPropagation
	 */
	public int getNbTotalPropagation() { return nbTotalPropagation;}
    
	/**
	 * Sets the number of propagation.
	 * @param nbTotalPropagation the number total of propagation.
	 */
	public void setNbTotalPropagation(int nbTotalPropagation) { this.nbTotalPropagation=nbTotalPropagation;}
    
	/**
	 * Gets the constraint names.
     * @return Vector
	 */
	public Vector getConstraintNames() { return constraintNames; }

    /** 
     * Sets the constraint names.
     * @param constraintNames 
     */
    public void setConstraintNames(Vector constraintNames){this.constraintNames=constraintNames;}

	/**
	 * Gets chrono of the propagation {@link #event} which ends this propagation process.
	 * @return int chrono
     * @see EventAttributes
	 */
	public int getChrono() { return event.getChrono();}

	
    /** 
     * Gets the {@link #type} of this MetaPropagationEvent.
     * 
     * @return int type of propagation event
     */
    public int getType(){return this.type;}

    /** 
     * Sets the type of this MetaPropagationEvent.
     * 
     * @param type int 
     * @see #type
     */
    public void setType(int type) {this.type=type;}
    
	/**
	 * Adds a typeEvent record for this event. That records
     * the number <i>nb</i> of times where <i>typeEvent</i> appears during this propagation phase,
     * and store the different constraints in <i>t</i> concerned. These are record in 
     * two hashtables {@link #totalEvents totalEvents} and {@link #filteredConstraints
     * filteredConstraints}.
	 * @param typeEvent the string which specifies the type of the {@link
     * EventAttributes} concerned.
	 * @param nb the int which specifies the number of time where <i>typeEvent</i>
     * appears during the current propagation process.
	 * @param t  a Hashtable wich contains constraints actived during the current propagation
     * process. This table can be null or not contains all constraints.
	 */
	public void addCounter(String typeEvent,int nb, Hashtable t){
		totalEvents.put(typeEvent,new Integer(nb));
		filteredConstraints.put(typeEvent,t);
	}

	/**
	 * Gets totalEvents counters.
	 * @return totalEvents the hashtable of all propagation events.
     * @see #totalEvents
	 */
	public Hashtable getTotalEvents(){ return totalEvents; }
	
	/**
	 * Gets filteredConstraints counters.
	 * @return filteredConstraints the hashtable of filtered constraints.
     * @see #filteredConstraints
	 */
	public Hashtable getFilteredConstraint(){ return filteredConstraints; }

}
